package hr.fer.bachelorThesis.algorithm.memory;

import hr.fer.bachelorThesis.algorithm.memory.MemoryElement;

/**
 * Map values.
 * 
 * @author Viran
 *
 */
public class MemoryElement implements Comparable<MemoryElement>, IMemoryElement {
	private EMemoryElement element;
	private int numberOfParticlesPassed;

	/**
	 * MapValues constructor.
	 * 
	 * @param element
	 *            Element value.
	 * @param numberOfParticlesPassed
	 *            Number of particles passed over this map value.
	 */
	public MemoryElement(EMemoryElement element) {
		this.element = element;
		this.numberOfParticlesPassed = 0;
	}

	@Override
	public EMemoryElement getElement() {
		return this.element;
	}

	@Override
	public int getNumOfPartPassed() {
		return numberOfParticlesPassed;
	}

	@Override
	public void incPass() {
		numberOfParticlesPassed++;
	}

	@Override
	public void setElement(EMemoryElement element) {
		this.element = element;
	}

	@Override
	public int compareTo(MemoryElement o) {
		return this.numberOfParticlesPassed - o.numberOfParticlesPassed;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		switch (element) {
		case UNEXPLORED:
			sb.append("UNEXPLORED");
			break;
		case EXIT:
			sb.append("EXIT");
			break;
		case PASSABLE:
			sb.append("PASSABLE");
			break;
		case WALL:
			sb.append("WALL");
			break;
		case START:
			sb.append("START");
			break;
		case TARGET:
			sb.append("TARGET");
			break;
		default:
			break;
		}
		sb.append(" Passed: " + numberOfParticlesPassed);
		return sb.toString();
	}
	
	@Override
	public String stringLabel() {
		switch (element) {
		case UNEXPLORED:
			return "U";
		case EXIT:
			return "E";
		case PASSABLE:
			return "P";
		case WALL:
			return "W";
		case START:
			return "S";
		case TARGET:
			return "T";
		default:
			break;
		}
		return "";
	}

	@Override
	public double calculatePositionScore() {
		if(this.element==EMemoryElement.EXIT)
			return 1e5;
		double stpNum = this.numberOfParticlesPassed;
		return 1 / (stpNum + 1);

	}
}
