package hr.fer.bachelorThesis.algorithm.memory;

import hr.fer.bachelorThesis.util.linearAlgebra.IVector;

public class PopulationMemory  implements ISharedMemory{
	
	protected MemoryElement[][] memory;
	protected int rows;
	protected int cols;

	public PopulationMemory(int rows, int cols) {
		this.rows=rows;
		this.cols=cols;
		this.memory=new MemoryElement[this.rows][this.cols];
		for(int i=0;i<rows;i++)
			for(int j=0;j<cols;j++)
				this.memory[i][j]=new MemoryElement(EMemoryElement.UNEXPLORED);
		
	}

	@Override
	public double steppedOn(IVector position) {
		MemoryElement steppedElement=memory[(int)position.get(0)][(int)position.get(1)];
		double positionScore=steppedElement.calculatePositionScore();
		if(steppedElement.getElement()==EMemoryElement.UNEXPLORED)
			steppedElement.setElement(EMemoryElement.PASSABLE);//TODO maybe add additional info to position
		steppedElement.incPass();
		return positionScore;
	}

	@Override
	public MemoryElement atPosition(IVector position) {
		MemoryElement memEle=memory[(int)position.get(0)][(int)position.get(1)];
		if(memEle.equals(EMemoryElement.UNEXPLORED))
			throw new IllegalArgumentException("Can not move to position"+((int)position.get(0))+" "+((int)position.get(0))+". Stepping on unexplored position is not possible.");
		if(memEle.equals(EMemoryElement.WALL))
			throw new IllegalArgumentException("Can not move to position"+((int)position.get(0))+" "+((int)position.get(0))+". That is a wall!");
		
		return memEle;
	}

	@Override
	public void discoveredPosition(IVector position, EMemoryElement element) {
		memory[(int)position.get(0)][(int)position.get(1)].setElement(element);
		
	}

	@Override
	public ISharedMemory copy() {
		return new PopulationMemory(this.rows, this.cols);
	}
	
}
