package hr.fer.bachelorThesis.algorithm.memory;

import java.util.Collections;
import java.util.LinkedList;

import hr.fer.bachelorThesis.util.linearAlgebra.IVector;
import hr.fer.bachelorThesis.util.linearAlgebra.IncompatibleOperandException;

public class TargetSharedMemory extends PopulationMemory implements ITargetSystemMemory {

	private class TargetRecord implements Comparable<TargetRecord> {
		private IVector position;
		private double positionPriority;

		public TargetRecord(IVector targetPosition, double targetFitness) {
			this.position = targetPosition;
			this.positionPriority = targetFitness;
		}

		@Override
		public int compareTo(TargetRecord arg0) {
			return Double.compare(this.positionPriority, arg0.positionPriority);
		}

		public IVector getPosition() {
			return position;
		}
	}

	private LinkedList<TargetRecord> targetsList = new LinkedList<>();
	
	
	public TargetSharedMemory(int rows, int cols) {
		super(rows, cols);
	}

	@Override
	public boolean addTarget(IVector targetPosition, double targetFitness) {
		//check if the target should be added and then add it 
		//1st see if the latest position has been reached at least once
		//...
		//just add the element and return here if necessary 
		this.targetsList.add(new TargetRecord(targetPosition, targetFitness));
		Collections.sort(this.targetsList);
		return true;
	}

	@Override
	public IVector getNextTarget(IVector position) {
		int index = 0;
		// see if you can fin the next target in list and return the next
		// element
		for (TargetRecord tr : this.targetsList) {
			if (position.equals(tr.getPosition())) {
				if (index + 1 == this.targetsList.size())
					return null;
				return this.targetsList.get(index + 1).getPosition();
			}
			index++;
		}
		// if that is not the case find the closest position to add the agent
		double minDistance = -1;
		int minIndex = 0;
		IVector distance = null;
		index=0;
		for (TargetRecord tr : this.targetsList) {
			try {
				distance = tr.getPosition().nSub(position);
			} catch (IncompatibleOperandException e) {
			}
			if (minDistance == -1) {
				minIndex = index;
				minDistance = distance.norm();
			}else if (distance.norm() <= minDistance) {
				minIndex = index;
				minDistance = distance.norm();
			}
			index++;
		}
		//return the position closest to the given point
		return this.targetsList.get(minIndex).getPosition();
	}

	
	@Override
	public String toString() {
		StringBuffer sb=new StringBuffer();
		
		for(int i=0;i<this.rows;i++){
			for(int j=0;j<this.cols;j++)
				sb.append(this.memory[i][j].stringLabel()+" ");
			sb.append("\n");
		}
		return sb.toString();
	}
}
