package hr.fer.bachelorThesis.algorithm.pso;

import java.util.concurrent.BlockingQueue;

import hr.fer.bachelorThesis.algorithm.IAlgorithmDrawer;
import hr.fer.bachelorThesis.algorithm.ISymulationSyncThread;
import hr.fer.bachelorThesis.controller.AgentSubIterationLog;
import hr.fer.bachelorThesis.controller.ThreadMonitor;
import hr.fer.bachelorThesis.controller.ThreadMonitor.Label;
import hr.fer.bachelorThesis.gui.labyrinth.LabyrinthView;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.paint.Color;

public class PSODrawer implements IAlgorithmDrawer, ISymulationSyncThread {

	private BlockingQueue<PSOSubIterationData> controllerDrawerQ;
	private double agentDim;
	private LabyrinthView labyrinthView;
	private ThreadMonitor monitor;
	private Color agentPaint;
	private Color bestAgentPaint;
	private long id;
	private Color memoryColor;
	private boolean displayMemory;
	//Map drawing data

	public PSODrawer(long id,BlockingQueue<PSOSubIterationData> algorithmOutputQueue, LabyrinthView labyrinthView, double agentDim,Color agentPaint,Color bestAgentPaint, boolean displayMemory) {
		this.id=id;
		this.controllerDrawerQ = algorithmOutputQueue;
		this.labyrinthView = labyrinthView;
		this.agentDim=agentDim;
		this.agentPaint=agentPaint;
		this.bestAgentPaint=bestAgentPaint;
		this.memoryColor=Color.color(agentPaint.getRed(), agentPaint.getGreen(), agentPaint.getBlue(),0.1);
		this.displayMemory=displayMemory;
	}

	@Override
	public void run() {
		while (true) {
			long timeStart=System.nanoTime();
			// exit mechanism
			if (this.checkMonitor() && this.controllerDrawerQ.isEmpty()){
				break;
			}

			PSOSubIterationData iterData = null; // TODO clean this a bit!

			try {
				iterData = controllerDrawerQ.take();
			} catch (InterruptedException e) {
				continue;
			}
			synchronized (labyrinthView) {
				this.labyrinthView.cleanAgentCanvas(id);
				GraphicsContext agentsPositionsLayer = this.labyrinthView.getAgentsPotsitionsLayer(id);
				GraphicsContext populationMemoryLayer = this.labyrinthView.getPopulationMemoryLayer(id);
				
				
				for (AgentSubIterationLog log : iterData.agentsLog) {
					// check id for agent colour
					if (log.agentId == iterData.bestAgentId) {
						agentsPositionsLayer.setFill(bestAgentPaint);
					} else {
						agentsPositionsLayer.setFill(agentPaint);
					}
					// TODO REFACTOR THIS! But not now! - back to 0,1??????
					 agentsPositionsLayer.fillOval(log.position.get(1)*this.agentDim,
					 log.position.get(0)*this.agentDim, this.agentDim,
					 this.agentDim);
					 
					 if(this.displayMemory){
						 populationMemoryLayer.setFill(this.memoryColor);
						 populationMemoryLayer.fillOval(log.position.get(1)*this.agentDim,
								 log.position.get(0)*this.agentDim, this.agentDim,
								 this.agentDim);	 
					 }
					 
				}
			}
			long timeEnd=System.nanoTime();
			double timeDiffMili=(timeEnd-timeStart)*1e-6; //nano to mili
			
			//double sleepTimeMilis = 1000 / 1;
			double sleepTimeMilis = 1000 / 24; // perhaps adding the hash map check
											// could be better
//			System.out.println("24 frames time:" +sleepTimeMilis);
//			System.out.println("Time diff: "+timeDiffMili);
			double sleepElapsedDifference=sleepTimeMilis-timeDiffMili;
			if(sleepElapsedDifference<=0){
				//just continue
			}else{

				try {
					Thread.sleep((long)sleepElapsedDifference);
					//Thread.sleep(5_000);
				} catch (InterruptedException e) {
					continue;
				}	
			}
		}
		
	}

	private boolean checkMonitor() {
		synchronized (monitor) {
			if (monitor.getLabel() == Label.PAUSE) {
				while (monitor.getLabel() == Label.PAUSE) {
					try {
						monitor.wait();
					} catch (InterruptedException e) {
					}
				}

				if (monitor.getLabel() == Label.STOP) {
					return true;
				} else if (monitor.getLabel() == Label.RUN) {
					
					return false;
				}
				
			} else if (monitor.getLabel() == Label.STOP) {
				return true;
			}
		}
		return false;
	}

	@Override
	public void setMonitor(ThreadMonitor monitor) {
		this.monitor = monitor;

	}
	
	@Override
	public double getAgentDim(){
		return this.agentDim;
	}
}
