package hr.fer.bachelorThesis.algorithm.stopCondition;

import hr.fer.bachelorThesis.algorithm.IPopulation;
import hr.fer.bachelorThesis.gui.ProgramGeneralConstants;
import javafx.scene.Node;
import javafx.scene.control.TextArea;
import javafx.scene.layout.ColumnConstraints;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.RowConstraints;

public class StopConditionFactory {

	public static enum EStopLabels {
		EXITS_FOUND_CONDITION, ITERATION_STOP_CONDITION, TIME_PASSED_CONDITION
	}

	public IStopCondition generateStopCondition(EStopLabels label, Object... params) {
		switch (label) {
		case EXITS_FOUND_CONDITION:
			return generateExitsFound(params);
		case ITERATION_STOP_CONDITION:
			return generateIterationStop(params);
		case TIME_PASSED_CONDITION:
			return generateTimePassed(params);
		default:
			return null;
		}
	}

	private IStopCondition generateTimePassed(Object[] params) {
		if (params.length != 1)
			throw new IllegalArgumentException(
					"Time passed condition needs 1 parameters: optimisation problem duration.");
		long durationTime = (Long) params[0];
		return new TimePassedCondition(durationTime);
	}

	private IStopCondition generateIterationStop(Object[] params) {
		if (params.length != 1)
			throw new IllegalArgumentException(
					"Iterations stop condition needs 1 parameters: maximal number of iterations.");
		int maxIterations = (Integer) params[0];
		return new IterationStopCondition(maxIterations);
	}

	private IStopCondition generateExitsFound(Object[] params) {
		if (params.length != 2)
			throw new IllegalArgumentException(
					"Exits found condition needs 2 parameters: population, and the number of exits to stop.");
		IPopulation pop = (IPopulation) params[0];
		int numOfExits = (Integer) params[1];
		return new ExitFoundCondition(pop, numOfExits);
	}

	public static StopConditionInputPane getStopConditonForm(String selectedGenerator) {

		if (selectedGenerator.equals(ProgramGeneralConstants.STOP_CONDITION_LABELS[0])) {
			// "Recursive random labyrinth"
			return new TimerForm();
		} else if (selectedGenerator.equals(ProgramGeneralConstants.STOP_CONDITION_LABELS[1])) {
			// "Fixed entrance labyrinth "
			return new IterationsForm();
		} else if (selectedGenerator.equals(ProgramGeneralConstants.STOP_CONDITION_LABELS[2])) {
			// "Fixed entrance labyrinth "
			return new ExitFoundForm();
		} 
		return null;

	}
}
