package hr.fer.bachelorThesis.algorithm.strategy;

import hr.fer.bachelorThesis.algorithm.IPopulation;
import hr.fer.bachelorThesis.algorithm.memory.ISharedMemory;
import hr.fer.bachelorThesis.labyrinth.ILabyrinth;

public class AgentStrategyFactory {

	public static enum EAgentStrategies{
		TARGET_DIRECTION,FAVOUR_UNVISITED,BEST_PREV,BEST_AGENT_DIRECTION, POPULATION_DIRECTION, NEIGHBOUR_DIRECTION, POPULATION_GROUPING
	}
	
	
	public IAgentStrategy generateStrategy(EAgentStrategies[] strategies,Object[][] params){
		IAgentStrategy compositStrategy=new CurrentDirection_BaseStrategy();
		
		if(strategies.length!=params.length) throw new IllegalArgumentException("Number of strategies and it's corresponding parametrs do not mach");
		
		for(int i=0;i<strategies.length;i++){
			EAgentStrategies strategy=strategies[i];
			Object[] strategyParams=params[i];
			
			switch(strategy){
			case BEST_AGENT_DIRECTION: compositStrategy=generateBestAgentDirStrat(compositStrategy,strategyParams); break;
			case BEST_PREV: compositStrategy=generateBestPrevDirStrat(compositStrategy,strategyParams); break;
			case FAVOUR_UNVISITED: compositStrategy=generateFavouUnexploredtrat(compositStrategy,strategyParams); break;
			case TARGET_DIRECTION: compositStrategy=generateTargetDirStrat(compositStrategy,strategyParams); break;
			case POPULATION_DIRECTION: compositStrategy=generatePopulationStrat(compositStrategy,strategyParams); break;
			case NEIGHBOUR_DIRECTION: compositStrategy=generateNeighbourStrat(compositStrategy,strategyParams); break;
			case POPULATION_GROUPING: compositStrategy=popGroupStrat(compositStrategy,strategyParams); break;
			}
		}
		
		return compositStrategy;
	}


	private IAgentStrategy popGroupStrat(IAgentStrategy compositStrategy, Object[] params) {
		if (params.length!=3) throw new IllegalArgumentException("Popultaion group streategy takes three parameter: population on which he operates, maximal population radius and the percentage of the maximal radius marked as a stable zone.");
		IPopulation population=(IPopulation)params[0];
		double maxPopulationRadius=(Double)params[1];
		double stableRangePercentage=(Double)params[2];
		return new PopulationGroupingStrategy(compositStrategy, population, maxPopulationRadius, stableRangePercentage);
	}


	private IAgentStrategy generateNeighbourStrat(IAgentStrategy compositStrategy, Object[] params) {
		if (params.length!=2) throw new IllegalArgumentException("Neighbour direction streategy takes two parameter: population on which he operates and maximal number of allowed steps in the desired diretion.");
		IPopulation population=(IPopulation)params[0];
		double maxVectrSize=(Double)params[1];
		
		return new NeighbourDirectionStrategy(compositStrategy, population,maxVectrSize);
	}


	private IAgentStrategy generatePopulationStrat(IAgentStrategy compositStrategy, Object[] params) {
		if (params.length!=2) throw new IllegalArgumentException("Population direction streategy takes two parameter: population on which he operates and maximal number of allowed steps in the desired diretion.");
		IPopulation population=(IPopulation)params[0];
		double maxVectrSize=(Double)params[1];
		
		return new PopulationDirectionStrategy(compositStrategy, population,maxVectrSize);
	}


	private IAgentStrategy generateTargetDirStrat(IAgentStrategy compositStrategy, Object[] params) {
		if (params.length!=1) throw new IllegalArgumentException("Target direction streategy takes exactly one parameter: maximal number of allowed steps in the desired diretion.");
		double maxVectrSize=(Double)params[0];
		return new TargetDirectionStrategy(compositStrategy, maxVectrSize);
	}


	private IAgentStrategy generateFavouUnexploredtrat(IAgentStrategy compositStrategy, Object[] params) {
		if (params.length!=3) throw new IllegalArgumentException("");
		ILabyrinth labyrinth=(ILabyrinth)params[0];
		ISharedMemory memory=(ISharedMemory)params[1];
		double unvisitedInfluence=(Double)params[2];
		return new FavourUnvisitedStrategy(labyrinth, memory, unvisitedInfluence, compositStrategy);
	}


	private IAgentStrategy generateBestPrevDirStrat(IAgentStrategy compositStrategy, Object[] params) {
		if (params.length!=1) throw new IllegalArgumentException("Best previous direction streategy takes exactly one parameter: maximal number of allowed steps in the desired diretion.");
		double maxVectrSize=(Double)params[0];
		return new BestPrevDirectionStrategy(compositStrategy, maxVectrSize);
	}


	private IAgentStrategy generateBestAgentDirStrat(IAgentStrategy compositStrategy, Object[] params) {
		if (params.length!=1) throw new IllegalArgumentException("Best previous direction streategy takes exactly one parameter: maximal number of allowed steps in the desired diretion.");
		double maxVectrSize=(Double)params[0];
		return new BestAgentDirectionStrategy(compositStrategy, maxVectrSize); //TODO potentional hazard!
	}
}
