package hr.fer.bachelorThesis.algorithm.strategy;

import hr.fer.bachelorThesis.algorithm.pso.IAgent;
import hr.fer.bachelorThesis.util.linearAlgebra.IVector;
import hr.fer.bachelorThesis.util.linearAlgebra.IncompatibleOperandException;

public class BestPrevDirectionStrategy implements IAgentStrategy {

	private IAgentStrategy strategy;
	private double maxVectorSize;
	
	public BestPrevDirectionStrategy(IAgentStrategy strategy,double maxVectorSize) {
		this.strategy=strategy;
		this.maxVectorSize=maxVectorSize;
	}
	
	@Override
	public IVector nextDirectionVector(IAgent agent) throws IncompatibleOperandException {
		if(agent.getBestPreviousPosition()!=null){
		
			//get the direction vector by subtracting the best position with the current position
			IVector thisStrategyVector=agent.getBestPreviousPosition().nSub(agent.getCurrentPosition()).normalise();
			//TODO perhaps scale it by a factor of previous best* current best...
			//Strategy:
			// 1.Make a direction vector from current to best position:
			// 2.Normalise it
			// 3.Multiply it by the (prevBestFit) value over (currentFitValue) the greater the distance, the bigger the desire to go back
			// 4.Multiply the solution with maxDistance
			
			double prevFitness=agent.getPrevBestFitness();
			if(prevFitness!=0){
				double value=agent.getFitness()/prevFitness;
				if(value>=1){
					value=1;
				}
				value=value*this.maxVectorSize;
				thisStrategyVector.scalarMultiply(value);	
			}else{
				if(thisStrategyVector.norm()>this.maxVectorSize)
					thisStrategyVector.normalise().scalarMultiply(this.maxVectorSize);
			}
			//System.out.println("Best prev. dir vector: "+thisStrategyVector);

			return strategy.nextDirectionVector(agent).add(thisStrategyVector);
			
		}else{
			return strategy.nextDirectionVector(agent);
			
		}
		
	}

}
