package hr.fer.bachelorThesis.algorithm.strategy;

import java.util.Collection;

import hr.fer.bachelorThesis.algorithm.IPopulation;
import hr.fer.bachelorThesis.algorithm.pso.IAgent;
import hr.fer.bachelorThesis.util.linearAlgebra.IVector;
import hr.fer.bachelorThesis.util.linearAlgebra.IncompatibleOperandException;
import hr.fer.bachelorThesis.util.linearAlgebra.Vector;

public class NeighbourDirectionStrategy implements IAgentStrategy {

	private IAgentStrategy strategy;
	private IPopulation population;
	private double maxVectorSize;

	public NeighbourDirectionStrategy(IAgentStrategy strategy, IPopulation population, double maxVectorSize) {
		this.strategy=strategy;
		this.population=population;
		this.maxVectorSize=maxVectorSize;
	}
	
	@Override
	public IVector nextDirectionVector(IAgent agent) throws IncompatibleOperandException {
		IVector dirVector=new Vector(0,0);
		Collection<IAgent> neighbours=population.getAgentNeighbours(agent);
		for(IAgent neighbour:neighbours)
			dirVector.add(neighbour.getCurrentDirectionVector());
		if(dirVector.norm()>maxVectorSize)
			dirVector.normalise().scalarMultiply(maxVectorSize);
		//System.out.println("Neighbour  dir vector: "+dirVector);
		return dirVector.add(strategy.nextDirectionVector(agent));
	}

}
