package hr.fer.bachelorThesis.algorithm.strategy;

import hr.fer.bachelorThesis.algorithm.IPopulation;
import hr.fer.bachelorThesis.algorithm.pso.IAgent;
import hr.fer.bachelorThesis.util.linearAlgebra.IVector;
import hr.fer.bachelorThesis.util.linearAlgebra.IncompatibleOperandException;
import hr.fer.bachelorThesis.util.linearAlgebra.Vector;
import hr.fer.bachelorThesis.util.random.IRNG;
import hr.fer.bachelorThesis.util.random.RNG;

public class PopulationGroupingStrategy implements IAgentStrategy {

	private IAgentStrategy strategy;
	private IPopulation population;
	private double maxPopulationRadius;
	private double minPopulationRadius;
	private IRNG rand=RNG.getRNG();
	
	public PopulationGroupingStrategy(IAgentStrategy strategy, IPopulation population, double maxPopulationRadius, double stableRangePercentage) {
		if(stableRangePercentage<0 && maxPopulationRadius>1)
			throw new IllegalArgumentException("stableRangePercentage must be a number between 0 and 1.");
		this.strategy=strategy;
		this.population=population;
		this.maxPopulationRadius=maxPopulationRadius;
		this.minPopulationRadius=maxPopulationRadius*(1-stableRangePercentage);
	}
	
	@Override
	public IVector nextDirectionVector(IAgent agent) throws IncompatibleOperandException {
		IVector popCenter=population.getCenterPosition();
		IVector agentCenterDistance=popCenter.nSub(agent.getCurrentPosition());
		double distance=agentCenterDistance.norm();
		IVector stabilityVector=new Vector(0,0);
		if(this.maxPopulationRadius>0){
			if(distance<=this.minPopulationRadius){
				//make the stability vector push the agent to the comfort zone
				stabilityVector=agentCenterDistance.normalise().scalarMultiply(-this.rand.nextDouble(0, maxPopulationRadius));
			}else  if(distance>this.maxPopulationRadius){
				//make the stability vector pull the agent to the comfort zone
				stabilityVector=agentCenterDistance.normalise().scalarMultiply(this.rand.nextDouble(0, maxPopulationRadius));
			}else{
				//leave the stability vector as it is: unmodified, set to (0,0)
			}

			//System.out.println("Grouping s dir vector: "+stabilityVector);

		}
		return stabilityVector.add(strategy.nextDirectionVector(agent));
	}

	
}
