package hr.fer.bachelorThesis.algorithm.util;

/**
 * Defines the real angle interval for the given discrete direction.
 * 
 * @author Viran
 *
 */
public class ParticleDirection {
	ELabyrinthDirection mainDirection;

	/**
	 * ParticleDirection constructor.
	 * 
	 * @param mainDirection
	 *            Direction.
	 */
	public ParticleDirection(ELabyrinthDirection mainDirection) {
		this.mainDirection = mainDirection;
	}

	/**
	 * Lower angle limit expressed in degrees. A positive angle is returned.
	 * 
	 * @return Lower limit.
	 */
	public double fromAngle() {

		switch (mainDirection) {
		case BASE:
			return 0;
		case EAST:
			return 71.57;
		case NORTH_EAST:
			return 108.43;
		case NORTH:
			return 161.57;
		case NORTH_WEST:
			return 198.43;
		case WEST:
			return 251.57;
		case SOUTH_WEST:
			return 288.43;
		case SOUTH:
			return 341.57;
		case SOUTH_EAST:
			return 18.43;
		default:
			return 0;
		}
	}

	/**
	 * Upper angle limit expressed in degrees. A positive angle is returned.
	 * 
	 * @return Upper limit.
	 */
	public double toAngle() {

		switch (mainDirection) {
		case BASE:
			return 0;
		case EAST:
			return 108.43;
		case NORTH_EAST:
			return 161.57;
		case NORTH:
			return 198.43;
		case NORTH_WEST:
			return 251.57;
		case WEST:
			return 288.43;
		case SOUTH_WEST:
			return 341.57;
		case SOUTH:
			return 18.43;
		case SOUTH_EAST:
			return 71.57;
		default:
			return 0;

		}
	}

	/**
	 * Convert this direction to x coordinate offset.
	 * 
	 * @return Integer representation of this coordinate .
	 */
	public int toXOffset() {

		switch (mainDirection) {
		case BASE:
			return 0;
		case EAST:
			return -1;
		case NORTH_EAST:
			return -1;
		case NORTH:
			return 0;
		case NORTH_WEST:
			return 1;
		case WEST:
			return 1;
		case SOUTH_WEST:
			return 1;
		case SOUTH:
			return 0;
		case SOUTH_EAST:
			return -1;

		default:
			return 0;
		}
	}

	/**
	 * Convert this direction to y coordinate offset.
	 * 
	 * @return Integer representation of this coordinate.
	 */
	public int toYOffset() {

		switch (mainDirection) {
		case BASE:
			return 0;
		case EAST:
			return 0;
		case NORTH_EAST:
			return -1;
		case NORTH:
			return -1;
		case NORTH_WEST:
			return -1;
		case WEST:
			return 0;
		case SOUTH_WEST:
			return 1;
		case SOUTH:
			return 1;
		case SOUTH_EAST:
			return 1;

		default:
			return 0;
		}
	}

	@Override
	public String toString() {
		return mainDirection.toString();
	}
}
