package hr.fer.bachelorThesis.controller;

import java.util.List;

import hr.fer.bachelorThesis.algorithm.IAlgorithmManager;
import hr.fer.bachelorThesis.controller.ThreadMonitor.Label;
import hr.fer.bachelorThesis.gui.IGUIController;

public class MainController implements IMainController {

	private IGUIController guiController;
	private List<IAlgorithmManager> algorithmManagers;
	private ThreadMonitor monitor;

	public MainController(IGUIController guiController, ThreadMonitor monitor,
			List<IAlgorithmManager> algorithmManagers, boolean sharedMemory) {
		this.guiController = guiController;
		this.algorithmManagers = algorithmManagers;
		this.monitor = monitor;

		if (sharedMemory) {
			IAlgorithmManager firstManager = algorithmManagers.get(0);
			for (int i = 1; i < algorithmManagers.size(); i++)
				algorithmManagers.get(i).setSharedMemory(firstManager);
		} // by default every algorithm has its own memory
	}

	@Override
	public void resume() {
		for (IAlgorithmManager manager : this.algorithmManagers)
			if (!manager.isDone())
				manager.resumeAlgorithm();
	}

	@Override
	public void pause() {
		for (IAlgorithmManager manager : this.algorithmManagers)
			if (!manager.isDone())
				manager.pauseAlgorithm();

	}

	@Override
	public void stop() {
		for (IAlgorithmManager manager : this.algorithmManagers)
			if (!manager.isDone())
				manager.stopAlgorithm();
	}

	@Override
	public void run() {
		// run all algorithms
		for (IAlgorithmManager manager : this.algorithmManagers)
			manager.startAlgorithm();

		while (true) {
			// flag set to stop
			if (checkMonitor())
				break;
			//all algorithms done
			int working=0;
			for(IAlgorithmManager manager: algorithmManagers)
				if (!manager.isDone())
					working++;
			if(working==0)
				break;
			//process iterations
			for (IAlgorithmManager manager : algorithmManagers)
				if (!manager.isDone())
					if (manager.drawingBufferEmpty())
						manager.prepareNextIteration();

		}

		this.stop();

	}

	private boolean checkMonitor() {
		synchronized (monitor) {

			if (monitor.getLabel() == Label.PAUSE) {
				this.pause();
				while (monitor.getLabel() == Label.PAUSE) {
					// first pause all other algorithms- then pause
					// yourself!
					try {
						monitor.wait();
					} catch (InterruptedException e) {
						System.out.println("FATAL ERROR.");
						System.exit(-1);
					}
				}

				if (monitor.getLabel() == Label.STOP) {
					this.stop();
					return true;
				} else if (monitor.getLabel() == Label.RUN) {
					this.resume();
					return false;
				}

			} else if (monitor.getLabel() == Label.STOP) {
				this.stop();
				return true;
			}

		}
		return false;
	}
}
