package hr.fer.bachelorThesis.gui;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import hr.fer.bachelorThesis.algorithm.IAlgorithmManager;
import hr.fer.bachelorThesis.controller.IMainController;
import hr.fer.bachelorThesis.controller.MainController;
import hr.fer.bachelorThesis.controller.ThreadMonitor;
import hr.fer.bachelorThesis.controller.ThreadMonitor.Label;
import hr.fer.bachelorThesis.gui.control.ConfigurationState;
import hr.fer.bachelorThesis.gui.control.IExecutionState;
import hr.fer.bachelorThesis.gui.control.PausedState;
import hr.fer.bachelorThesis.gui.control.RunningState;
import hr.fer.bachelorThesis.gui.labyrinth.LabyrinthView;
import hr.fer.bachelorThesis.labyrinth.ILabyrinth;
import javafx.scene.layout.Pane;

public class GUIController implements IGUIController {

	// Controller states
	private IExecutionState runningState;
	private IExecutionState pausedState;
	private IExecutionState configurationState;
	private IExecutionState currentState;

	// Utility objects
	private Pane changableUIPane;
	private HashMap<String, Object> controllerMap;
	private LabyrinthView labyrinthView;

	// simulation controller
	private IMainController mainController;
	private Thread mainControllerThread;
	private ThreadMonitor monitor;
	private ILabyrinth labyrinth;
	
	public GUIController() {
		
		
		// utility initialise
		this.controllerMap = new HashMap<String, Object>();

		// states initialise
		this.runningState = new RunningState(this);
		this.pausedState = new PausedState(this);
		this.configurationState = new ConfigurationState(this);
		this.currentState = this.configurationState;
		
	}

	@Override
	public Map<String, Object> getControlelrMap() {
		return controllerMap;
	}

	@Override
	public void startSimulation(List<IAlgorithmManager> managers) {
		if(managers.size()==0) return;
		//WARNING! This code must run
		for(IAlgorithmManager manager:managers)
			manager.setLabyrinthView(this.labyrinthView, labyrinth.getBlockHeight());
		//---------------------------
		
		// TODO start the main controller
		this.currentState = this.runningState;
		// remove all children and add new pane
		this.changableUIPane.getChildren().clear();
		this.changableUIPane.getChildren().add(this.currentState.getUIPane());
		// TODO Auto-generated method stub
		this.monitor=new ThreadMonitor(Label.RUN);
		this.mainController = new MainController(this,this.monitor,managers, false); //TODO add a button to controll shared memory!!!! 
		this.mainControllerThread=new Thread(mainController);
		this.mainControllerThread.start();
	}

	@Override
	public void pauseSimulation() {
		// TODO Auto-generated method stub
		// TODO call pause method on main controller
		this.currentState = this.pausedState;
		// remove all children and add new pane
		this.changableUIPane.getChildren().clear();
		this.changableUIPane.getChildren().add(this.currentState.getUIPane());
		
		//propagate
		//this.mainController.pause(); TODO!!!!!!!! maybe thiw won't work?
		synchronized (monitor) {
			monitor.setLabel(Label.PAUSE);
		}
	}
	
	@Override
	public void resumeSimulation() {
		// TODO start the main controller
		this.currentState = this.runningState;
		// remove all children and add new pane
		this.changableUIPane.getChildren().clear();
		this.changableUIPane.getChildren().add(this.currentState.getUIPane());
		// TODO Auto-generated method stub
		
		//propagate
		synchronized (monitor) {
			monitor.setLabel(Label.RUN);
			monitor.notify();
		}
	}

	@Override
	public void stopSimulation() {
		// TODO stop main controller and set the proper state
		// TODO call stop method on the main controller
		this.currentState = this.configurationState;
		this.changableUIPane.getChildren().clear();
		this.changableUIPane.getChildren().add(this.currentState.getUIPane());
		// remove all children and add new pane

		synchronized (monitor) {
			monitor.setLabel(Label.STOP);
			monitor.notify();
			//mainController.stop();
		}
		try {
			this.mainControllerThread.join(); //TODO ! check if this holds!
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		this.labyrinthView.clearAllAlgorithmLayers();
	}

	@Override
	public void setLabyrinthView(LabyrinthView labyrinthView) {
		this.labyrinthView = labyrinthView;
		if(this.labyrinth!=null)
			this.labyrinthView.setLabyrinth(this.labyrinth);
	}

	@Override
	public LabyrinthView getLabyrinthView() {
		return this.labyrinthView;
	}

	@Override
	public void setLabyrinth(ILabyrinth labyrinth) {
		this.labyrinth=labyrinth;
		if(this.labyrinthView!=null){
			this.labyrinthView.setLabyrinth(labyrinth);
		}
		
	}

	@Override
	public void setInteractionPane(Pane changableUIPane) {
		this.changableUIPane=changableUIPane;
		this.currentState = this.configurationState;
		this.changableUIPane.getChildren().clear();
		this.changableUIPane.getChildren().add(this.currentState.getUIPane());
	}


}
