package hr.fer.bachelorThesis.gui;

import java.awt.BorderLayout;

import hr.fer.bachelorThesis.gui.labyrinth.LabyrinthView;
import hr.fer.bachelorThesis.gui.other.AboutWindow;
import hr.fer.bachelorThesis.gui.other.HelpWindow;
import javafx.geometry.Insets;
import javafx.scene.control.Menu;
import javafx.scene.control.MenuBar;
import javafx.scene.control.MenuItem;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.ColumnConstraints;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.Pane;
import javafx.scene.layout.RowConstraints;
import javafx.stage.Stage;

/**
 * Class for building a window used in laybirnigh simulation.
 * @author Viran
 *
 */
public class WindowBuilder {

	/**
	 * Main method for building the window.
	 * @param controller COntroller used for this window.
	 * @return GridPane layout containing window elements.
	 */
	public  GridPane buildWindow(IGUIController controller){
		GridPane layout=new GridPane();
		return mainPanelLayout(layout,controller);
	}
	
	
	/**
	 * Generates main layout for this window.
	 * The window is first divided into two sections: the menu bar on top and the application display on the bottom.
	 * @return GridPane object containing the layout of the windows.
	 */
	private  GridPane mainPanelLayout(GridPane layout,IGUIController controller) {
		// Main panel layout
		//styleMainPanelLayout(layout);
		//layout.add(menuBar(), 0, 0);
		layout.add(mainDisplay(controller), 0, 0);
		
		return layout;
	}

	/**
	 * Method for styling the main panel layout.
	 * @param layout Layout of the main panel.
	 */
	private  void styleMainPanelLayout(GridPane layout) {
		layout.setHgap(4);
		layout.setVgap(4);
		layout.setPadding(new Insets(4, 4, 4, 0));
		//layout.setPrefHeight(ProgramGeneralConstants.INITIAL_WINDOW_HEIGHT);
		//layout.setPrefWidth(ProgramGeneralConstants.INITIAL_WINDOW_WIDTH);

		// add one column
		ColumnConstraints column = new ColumnConstraints();
		column.setPercentWidth(100);
		layout.getColumnConstraints().add(column);
		// add two rows
		RowConstraints row1 = new RowConstraints();
		layout.getRowConstraints().add(row1);
		RowConstraints row2 = new RowConstraints();
		layout.getRowConstraints().add(row2);

		
	}

	/**
	 * Window partition containing the simulation screen as well as the rest of the 
	 * @return
	 */
	private  BorderPane mainDisplay(IGUIController controller) {
		//System.err.println("Building main display");

		// Main display layout
		GridPane layout = new GridPane();
		//styleMainDisplay(layout);
		BorderPane borderPane = new BorderPane();
		LabyrinthView selectedView=new LabyrinthView(controller, ProgramGeneralConstants.INITIAL_CANVAS_HEIGHT,
				ProgramGeneralConstants.INITIAL_CANVAS_WIDTH);
		controller.setLabyrinthView(selectedView);
		Pane changableUIPane= new Pane();
		controller.setInteractionPane(changableUIPane);
		layout.add(selectedView,0,0);
		layout.add(changableUIPane, 1, 0);
		borderPane.setLeft(selectedView);
		borderPane.setRight(changableUIPane);
		return borderPane;
	}

	/**
	 * 
	 * @param layout
	 */
	private  void styleMainDisplay(GridPane layout) {
		layout.setHgap(4);
		layout.setVgap(4);
		layout.setPadding(new Insets(4, 4, 4, 0));

		// add two columns
		ColumnConstraints column1 = new ColumnConstraints();
		column1.setPercentWidth(75);
		layout.getColumnConstraints().add(column1);

		ColumnConstraints column2 = new ColumnConstraints();
		column2.setPercentWidth(25);
		layout.getColumnConstraints().add(column2);

		// add one row
		RowConstraints row = new RowConstraints();
		layout.getRowConstraints().add(row);
		
	}

	/**
	 * 
	 * @return
	 */
	private  MenuBar menuBar() {
		//System.err.println("Building menu bar");
		MenuBar mainMenuBar = new MenuBar();
		//TODO maybe this is good! mainMenuBar.setPrefWidth(ProgramGeneralConstants.INITIAL_WINDOW_WIDTH);
		Menu menu = new Menu();
		menu.setText("Menu");
		mainMenuBar.getMenus().add(menu);

		MenuItem about = new MenuItem();
		MenuItem help = new MenuItem();
		menu.getItems().addAll(about, help);
		
		about.setText("About");
		about.setOnAction((event)->{
			Stage aboutWindow = new AboutWindow();
			aboutWindow.show();
		});
		
		help.setText("Help");
		help.setOnAction((event)->{
			Stage helpWindow=new HelpWindow();
			helpWindow.show();
		});
		
		return mainMenuBar;
	}
	
}
