package hr.fer.bachelorThesis.gui.algorithm;

import java.util.LinkedList;
import java.util.List;

import hr.fer.bachelorThesis.algorithm.IAlgorithmManager;
import hr.fer.bachelorThesis.gui.ProgramGeneralConstants;
import hr.fer.bachelorThesis.gui.control.ConfigurationPane;
import hr.fer.bachelorThesis.gui.control.StopConditionPane;
import hr.fer.bachelorThesis.labyrinth.ILabyrinth;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonBar;
import javafx.scene.control.ComboBox;
import javafx.scene.control.TitledPane;
import javafx.scene.control.ToggleButton;
import javafx.scene.layout.ColumnConstraints;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.RowConstraints;

public class AlgorithmPane extends TitledPane {

	private static GridPane layout = new GridPane();

	private AlgorithmInputPane currentAlgorithmForm;
	private ToggleButton alg1Toggle;
	private ToggleButton alg2Toggle;
	
	private IAlgorithmManager[] algorithmManagersList;
	//private IAlgorithmManager algorithmManagers1;
	//private IAlgorithmManager algorithmManagers2;
	
	

	private ILabyrinth labyrinth;
	
	public AlgorithmPane(ConfigurationPane configurationPane, StopConditionPane stopConditionPane) {
		super("Optimisation algorithms", layout);
		buildOptAlgorithmsPane(this, configurationPane,stopConditionPane);
		this.algorithmManagersList=new IAlgorithmManager[2];
	}

	private void buildOptAlgorithmsPane(AlgorithmPane algorithmPane, ConfigurationPane configurationPane,StopConditionPane stopConditionPane) {
		//styleOptAlgorithmsPane(layout);

		algorithmPane.setExpanded(false);
		algorithmPane.setCollapsible(true);

		ComboBox<String> selGenComboBox = new ComboBox<>();
		selGenComboBox.getItems().addAll(ProgramGeneralConstants.OPTIMISATION_ALGORITHMS_LABELS);
		selGenComboBox.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(ActionEvent event) {
				// Provide parameters form
				String selectedGenerator = selGenComboBox.getSelectionModel().getSelectedItem();
				if(currentAlgorithmForm!=null)
					layout.getChildren().remove(currentAlgorithmForm);
				currentAlgorithmForm=AlgorithmFormFactory.getAlgorithmForm(selectedGenerator,stopConditionPane);
				currentAlgorithmForm.setLabyrinth(labyrinth);
				layout.add(currentAlgorithmForm, 0, 1);
			}
		});
		layout.add(selGenComboBox, 0, 0);
		Button selectGenerator=new Button("Add algorithm");
		selectGenerator.setOnAction((event)->{
			if(this.alg1Toggle.isDisable()){
				this.generateNextAlgorithm(0);
				this.alg1Toggle.setDisable(false);
				return;
			}
			if(this.alg2Toggle.isDisable()){
				this.generateNextAlgorithm(1);
				this.alg2Toggle.setDisable(false);
				return;
			}
		});
		this.alg1Toggle= new ToggleButton("Algorithm 1");
		this.alg1Toggle.setDisable(true);
		this.alg1Toggle.setSelected(false);
		this.alg1Toggle.setOnAction((event)->{
			this.removeAlgorithm(0);
			this.alg1Toggle.setDisable(true);
			this.alg1Toggle.setSelected(false);
		});
		
		this.alg2Toggle= new ToggleButton("Algorithm 2");
		this.alg2Toggle.setDisable(true);
		this.alg2Toggle.setSelected(false);
		this.alg2Toggle.setOnAction((event)->{
			this.removeAlgorithm(1);
			this.alg2Toggle.setDisable(true);
			this.alg2Toggle.setSelected(false);
		});

		layout.add(alg1Toggle, 0, 2);
		layout.add(alg2Toggle, 0, 3);
		layout.add(selectGenerator, 0, 4);
	}

	private void removeAlgorithm(int i) {
		this.algorithmManagersList[i]=null;		
	}

	public AlgorithmInputPane getActiveForm() {
		return currentAlgorithmForm;
	}

	public void generateNextAlgorithm(int i) {
		this.algorithmManagersList[i]=this.currentAlgorithmForm.generateNextAlgorithm();		
	}

	public void setLabyrinth(ILabyrinth labyrinth) {
		this.labyrinth=labyrinth;
		if(currentAlgorithmForm!=null)
			currentAlgorithmForm.setLabyrinth(labyrinth);

	}

	public List<IAlgorithmManager> getAlgorithmManagers() {
		List<IAlgorithmManager> list=new LinkedList<>();
		if(this.algorithmManagersList[0]!=null){
			list.add(this.algorithmManagersList[0]);
			this.algorithmManagersList[0].getAlgorithm().setID(0);
		}
		if(this.algorithmManagersList[1]!=null){
			list.add(this.algorithmManagersList[1]);
			this.algorithmManagersList[1].getAlgorithm().setID(1);
		}
		this.alg1Toggle.setDisable(true);
		this.alg1Toggle.setSelected(false);
		this.alg2Toggle.setDisable(true);
		this.alg2Toggle.setSelected(false);
		this.algorithmManagersList=new IAlgorithmManager[2];
		return list;
//		List<IAlgorithmManager> list=this.algorithmManagersList;
//		this.algorithmManagersList=new LinkedList<>();
//		return list;
	}

}
