package hr.fer.bachelorThesis.gui.algorithm;

import hr.fer.bachelorThesis.algorithm.AlgorithmIdGenerator;
import hr.fer.bachelorThesis.algorithm.IAlgorithmManager;
import hr.fer.bachelorThesis.algorithm.IPopulation;
import hr.fer.bachelorThesis.algorithm.memory.ITargetSystemMemory;
import hr.fer.bachelorThesis.algorithm.memory.TargetSharedMemory;
import hr.fer.bachelorThesis.algorithm.pso.PSOAlgorithm;
import hr.fer.bachelorThesis.algorithm.pso.PSOAlgorithmManager;
import hr.fer.bachelorThesis.algorithm.pso.Population;
import hr.fer.bachelorThesis.algorithm.stopCondition.IStopCondition;
import hr.fer.bachelorThesis.algorithm.strategy.AgentStrategyFactory;
import hr.fer.bachelorThesis.algorithm.strategy.IAgentStrategy;
import hr.fer.bachelorThesis.algorithm.strategy.AgentStrategyFactory.EAgentStrategies;
import hr.fer.bachelorThesis.gui.control.StopConditionPane;
import hr.fer.bachelorThesis.labyrinth.ILabyrinth;
import javafx.scene.control.ColorPicker;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollBar;
import javafx.scene.control.TextField;
import javafx.scene.layout.GridPane;
import javafx.scene.paint.Color;

public class PSOForm extends AlgorithmInputPane  {

	//variables
	static private final int numberOfAgents = 5;
	static private final double maxAgentSpeed = 10.;
	static private final int neighbourCount = 2;
	
	//Strategy parameters
	private static final double MIN_INFLUENCE = 0;
	private static final double MAX_INFLUENCE = 1;
	
	private static final double bestAgentInfluanceVal = 0.15;
	private static final double bestPrevInfluanceVal = 0.08;
	private static final double unvisitedInfluanceVal = 0.3;
	private static final double populationInfluanceVal = 0.15;
	private static final double neighbourInfluanceVal = 0.15;

	private static final double maxPopRadius = 0;
	private static final double comfZonePerc = 100;
	
	
	private Color defaultAgentColor=Color.BLUE;
	private Color defaultBestAgentColor=Color.AZURE;
	
	// graphical components
	private TextField numberOfAgentsInput;
	private TextField maxAgentSpeedInput;
	private TextField neighboursPerAgentInput;
	private ScrollBar bestAgentInfluanceInput;
	private ScrollBar bestPreviousPositionInfluanceInput;
	private ScrollBar targetInfluanceInput;
	private ScrollBar unvisitedInfluanceInput;
	private ScrollBar populationInfluanceInput;
	private ScrollBar neighbourhoodInfluanceInput;
	private TextField comfortZonePercentageInput;
	private TextField maxPopRadiusInput;
	private ColorPicker agentColorPicker;
	private ColorPicker bestAgentColorPicker;
	
	private GridPane layout;
	private StopConditionPane stopConditionPane;
	private ILabyrinth labyrinth;

	public PSOForm() {
		this.layout=new GridPane();
		this.getChildren().add(this.layout);
		initialise();
	}
	
	public PSOForm(StopConditionPane stopConditionPane) {
		this.layout=new GridPane();
		this.stopConditionPane=stopConditionPane;
		this.getChildren().add(layout);
		initialise();
	}

	private void initialise() {

		this.layout.add(new Label("Number of agents: "), 0, 0);
		numberOfAgentsInput = new TextField();
		numberOfAgentsInput.setText(String.valueOf(numberOfAgents));
		this.layout.add(numberOfAgentsInput, 1, 0);

		this.layout.add(new Label("Maximal agent speed: "), 0, 1);
		maxAgentSpeedInput = new TextField();
		maxAgentSpeedInput.setText(String.valueOf(maxAgentSpeed));
		this.layout.add(maxAgentSpeedInput, 1, 1);

		this.layout.add(new Label("Neighbours per agent: "), 0, 2);
		neighboursPerAgentInput = new TextField();
		neighboursPerAgentInput.setText(String.valueOf(neighbourCount));
		this.layout.add(neighboursPerAgentInput, 1, 2);

		//--------------------------------------
		this.layout.add(new Label("Best agent influence: "), 0, 3);
		bestAgentInfluanceInput = new ScrollBar();
		bestAgentInfluanceInput.setMin(MIN_INFLUENCE);
		bestAgentInfluanceInput.setMax(MAX_INFLUENCE);
		//bestAgentInfluanceInput.setValue(bestAgentInfluanceVal);
		bestAgentInfluanceInput.setValue(bestAgentInfluanceVal);
		bestAgentInfluanceInput.setUnitIncrement(1);
		bestAgentInfluanceInput.setBlockIncrement(1);
		this.layout.add(bestAgentInfluanceInput, 0, 4, 2, 1); //TODO ??? what does this look like
		
		this.layout.add(new Label("Best previous position influence: "), 0, 5);
		bestPreviousPositionInfluanceInput = new ScrollBar();
		bestPreviousPositionInfluanceInput.setMin(MIN_INFLUENCE);
		bestPreviousPositionInfluanceInput.setMax(MAX_INFLUENCE);
		//bestPreviousPositionInfluanceInput.setValue(bestPrevInfluanceVal);
		bestPreviousPositionInfluanceInput.setValue(bestPrevInfluanceVal);
		
		bestPreviousPositionInfluanceInput.setUnitIncrement(1);
		bestPreviousPositionInfluanceInput.setBlockIncrement(1);
		this.layout.add(bestPreviousPositionInfluanceInput, 0, 6, 2, 1); //TODO ??? what does this look like
		
//		this.layout.add(new Label("Next target influence: "), 0, 7);
//		targetInfluanceInput = new ScrollBar();
//		targetInfluanceInput.setMin(MIN_INFLUENCE);
//		targetInfluanceInput.setMax(MAX_INFLUENCE);
//		//targetInfluanceInput.setValue(targetInfluanceVal);
//		targetInfluanceInput.setValue(defaultInfluanceVal);
//		
//		targetInfluanceInput.setUnitIncrement(1);
//		targetInfluanceInput.setBlockIncrement(1);
//		this.layout.add(targetInfluanceInput, 0, 8, 2, 1); //TODO ??? what does this look like
		
		this.layout.add(new Label("Unexplored position influence: "), 0, 7);
		unvisitedInfluanceInput = new ScrollBar();
		unvisitedInfluanceInput.setMin(MIN_INFLUENCE);
		unvisitedInfluanceInput.setMax(MAX_INFLUENCE);
		//unvisitedInfluanceInput.setValue(unvisitedInfluanceVal);
		unvisitedInfluanceInput.setValue(unvisitedInfluanceVal);
		
		unvisitedInfluanceInput.setUnitIncrement(1);
		unvisitedInfluanceInput.setBlockIncrement(1);
		this.layout.add(unvisitedInfluanceInput, 0,8, 2, 1); //TODO ??? what does this look like
		
		this.layout.add(new Label("Population influence: "), 0, 9);
		populationInfluanceInput = new ScrollBar();
		populationInfluanceInput.setMin(MIN_INFLUENCE);
		populationInfluanceInput.setMax(MAX_INFLUENCE);
		//populationInfluanceInput.setValue(populationInfluanceVal);
		populationInfluanceInput.setValue(populationInfluanceVal);
		
		populationInfluanceInput.setUnitIncrement(1);
		populationInfluanceInput.setBlockIncrement(1);
		this.layout.add(populationInfluanceInput, 0, 10, 2, 1); //TODO ??? what does this look like
		
		this.layout.add(new Label("Neighbour influence: "), 0, 11);
		neighbourhoodInfluanceInput = new ScrollBar();
		neighbourhoodInfluanceInput.setMin(MIN_INFLUENCE);
		neighbourhoodInfluanceInput.setMax(MAX_INFLUENCE);
		//neighbourhoodInfluanceInput.setValue(neighbourInfluanceVal);
		neighbourhoodInfluanceInput.setValue(neighbourInfluanceVal);
		
		neighbourhoodInfluanceInput.setUnitIncrement(1);
		neighbourhoodInfluanceInput.setBlockIncrement(1);
		this.layout.add(neighbourhoodInfluanceInput, 0, 12, 2, 1); //TODO ??? what does this look like
		
		this.layout.add(new Label("Maximal population radius: "), 0, 13);
		maxPopRadiusInput = new TextField();
		maxPopRadiusInput.setText(String.valueOf(maxPopRadius));
		this.layout.add(maxPopRadiusInput, 1, 13);
		
		this.layout.add(new Label("Comfort zone percentage: "), 0,14);
		comfortZonePercentageInput = new TextField();
		comfortZonePercentageInput.setText(String.valueOf(comfZonePerc));
		this.layout.add(comfortZonePercentageInput, 1, 14);
		
		this.layout.add(new Label("Agent color:"), 0,15);
		agentColorPicker=new ColorPicker(defaultAgentColor);
		this.layout.add(agentColorPicker, 1, 15);
		
		this.layout.add(new Label("Best color:"), 0, 16);
		bestAgentColorPicker=new ColorPicker(defaultBestAgentColor);
		this.layout.add(bestAgentColorPicker, 1, 16);
		
	}
	
	public int getNumberOfAgentsInput() {
		int number= Integer.parseInt(numberOfAgentsInput.getText());
		if(number<=0)
			return numberOfAgents;
		else
			return number;
	}
	
	public double getMaxAgentSpeedInput() {
		double speed=Double.parseDouble(maxAgentSpeedInput.getText());
		if(speed<=0)
			return maxAgentSpeed;
		else
			return speed;
	}
	
	public int getNeighboursPerAgentInput() {
		int numberOfAgents=this.getNumberOfAgentsInput();
		int neigh=Integer.parseInt(neighboursPerAgentInput.getText());
		if(neigh<0)
			return 0;
		else if(neigh>numberOfAgents)
			return numberOfAgents;
		else 
			return neigh;
	}

	public double getMaxInfluanceBestAgent(){
		return bestAgentInfluanceInput.getValue();
	}
	
	public double getMaxInfluanceBestPrevPosition(){
		return bestPreviousPositionInfluanceInput.getValue();
	}
	
	public double getMaxInfluanceTargetDirecton(){
		return targetInfluanceInput.getValue();
	}
	
	public double getMaxUnvisitedInfluence(){
		return unvisitedInfluanceInput.getValue();
	}
	
	private double getMaxNeighbourhoodInfluence() {
		return populationInfluanceInput.getValue();
	}

	private double getMaxPopulationInfluence() {
		return neighbourhoodInfluanceInput.getValue();
	}
	
	private double getComfortZonePercentage() {
		double percentage= Double.parseDouble(comfortZonePercentageInput.getText());
		if(percentage<0 || percentage>1)
			return 1;
		else
			return percentage/100;
	}

	private double getMaxPopulationRadius() {
		double maxRadius= Double.parseDouble(maxPopRadiusInput.getText());
		if(maxRadius<=0)
			return maxPopRadius;
		else
			return maxRadius;
	}
	

	private Color getAgentColor() {
		return agentColorPicker.getValue();
	}

	private Color getGlobalBestColor() {
		return bestAgentColorPicker.getValue();
	}


	
	@Override
	public void setLabyrinth(ILabyrinth labyrinth) {
		this.labyrinth=labyrinth;
	}

	@Override
	public IAlgorithmManager generateNextAlgorithm() {
		double maxAgentSpeed=this.getMaxAgentSpeedInput();
		IPopulation population=new Population(this.getNumberOfAgentsInput(), labyrinth.getStartingPosition(), maxAgentSpeed, this.getMaxAgentSpeedInput(), this.getNeighboursPerAgentInput());
		//generate strategies
	
		ITargetSystemMemory sharedMemory=new TargetSharedMemory(labyrinth.getRowCount(), labyrinth.getColumnCount());
		EAgentStrategies[] strategies= new EAgentStrategies[]{ EAgentStrategies.BEST_AGENT_DIRECTION, EAgentStrategies.BEST_PREV,  EAgentStrategies.FAVOUR_UNVISITED,EAgentStrategies.POPULATION_DIRECTION,EAgentStrategies.NEIGHBOUR_DIRECTION, EAgentStrategies.POPULATION_GROUPING};
		Object[][] params= new Object[][]{
				new Object[]{ maxAgentSpeed*this.getMaxInfluanceBestAgent() },
				new Object[]{ maxAgentSpeed*this.getMaxInfluanceBestPrevPosition() },
				new Object[] { labyrinth, sharedMemory, maxAgentSpeed*this.getMaxUnvisitedInfluence()},
				new Object[] { population, maxAgentSpeed*this.getMaxPopulationInfluence()},
				new Object[] { population, maxAgentSpeed*this.getMaxNeighbourhoodInfluence()},
				new Object[] { population, this.getMaxPopulationRadius(),this.getComfortZonePercentage()},
				
		};
		
		IAgentStrategy agentStrategy= new AgentStrategyFactory().generateStrategy(strategies, params);
		//generate stop condition
		IStopCondition stopCondition= stopConditionPane.generateStopCondition(population);
		//build algorithm
		PSOAlgorithm algorithm= new PSOAlgorithm(AlgorithmIdGenerator.getUniqueAlgorithmId(), labyrinth, agentStrategy , population, stopCondition);
		algorithm.setSharedMemory(sharedMemory);
		return new PSOAlgorithmManager(algorithm, this.getAgentColor(), this.getGlobalBestColor());
		
	}

}
