package hr.fer.bachelorThesis.gui.control;


import java.util.List;

import hr.fer.bachelorThesis.algorithm.IAlgorithmManager;
import hr.fer.bachelorThesis.gui.IGUIController;
import hr.fer.bachelorThesis.gui.algorithm.AlgorithmConfigurationPane;
import hr.fer.bachelorThesis.gui.labyrinth.LabyrinthPane;
import hr.fer.bachelorThesis.labyrinth.ILabyrinth;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.control.Accordion;
import javafx.scene.control.Button;
import javafx.scene.control.CheckBox;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.Pane;

public class ConfigurationPane extends Pane {
	
	private LabyrinthPane labyrinthGeneratorPane;
	private AlgorithmConfigurationPane algConfPane;
	//private ButtonBar buttonBar;
	private IGUIController guiController;
	
	public ConfigurationPane(IGUIController guiController) {
		this.guiController=guiController;
		this.getChildren().add(setupDisplay(guiController));
		
	}

	private BorderPane setupDisplay(IGUIController guiController) {
		// System.err.println("Building setup view");
		BorderPane layout = new BorderPane();
		//styleSetupDisplay(layout);
		Accordion accordition = new Accordion();

		
		layout.setCenter(accordition);

		this.labyrinthGeneratorPane = new LabyrinthPane(this);// buildLabGeneratorPane();
		this.algConfPane = new AlgorithmConfigurationPane(this);

		//re-enable only when the labyrinth is selected
		this.algConfPane.setDisable(true);
		
		accordition.getPanes().addAll(this.labyrinthGeneratorPane, this.algConfPane); // runConfigTitledPane);
		accordition.setExpandedPane(this.labyrinthGeneratorPane);


		Button startSimButton = new Button("Start simulation");

		CheckBox alg1MemToggle=new CheckBox("Display population memory:");
		alg1MemToggle.setSelected(true);
		startSimButton.setOnAction(new EventHandler<ActionEvent>() {

			@Override
			public void handle(ActionEvent event) {
				List<IAlgorithmManager> algMList=getAlgorithmManagers();
				
				for(IAlgorithmManager alM:algMList)
					alM.shouldDisplayMemory(alg1MemToggle.isSelected());
				
				guiController.startSimulation(algMList);
			}

		});
		
		GridPane startingPanel=new GridPane();
		startingPanel.add(alg1MemToggle, 0, 0);
		startingPanel.add(startSimButton, 0, 1);
		layout.setBottom(startingPanel);
		
		return layout;
	}

	private List<IAlgorithmManager> getAlgorithmManagers() {
		return this.algConfPane.getAlgorithmManagers();
	}

	public void setLabyrinthSelected(ILabyrinth labyrinth){
		this.guiController.setLabyrinth(labyrinth);
		this.algConfPane.setDisable(false);
		this.algConfPane.setLabyrinth(labyrinth);
		
	}


	public int getCanvasDimension() {
		return guiController.getLabyrinthView().getCanvasWidth();
		
	}


}
