package hr.fer.bachelorThesis.gui.labyrinth;

import hr.fer.bachelorThesis.labyrinth.generator.EComplexity;
import hr.fer.bachelorThesis.labyrinth.generator.EEntranceLabel;
import javafx.geometry.Insets;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollBar;
import javafx.scene.control.TextField;
import javafx.scene.layout.ColumnConstraints;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.Pane;
import javafx.scene.layout.RowConstraints;

public class FELabForm extends Pane implements IDomainForm{

	public class FELabParameters implements IDomainParameters{

		public int labSize;
		public EEntranceLabel entranceLabel;
		public EComplexity complexity;
		public int numberOfExits;
		
		public FELabParameters(int labSize,EEntranceLabel entranceLabel,EComplexity complexity,int numberOfExits) {
			this.labSize=labSize;
			this.entranceLabel=entranceLabel;
			this.complexity=complexity;
			this.numberOfExits=numberOfExits;
		}
		
		@Override
		public IDomainParameters copy() {
			return new FELabParameters(this.labSize,this.entranceLabel,this.complexity,this.numberOfExits);
		}

	}
	
	static final private int MIN_DOMAIN_SIZE = 2;
	static final private int MAX_DOMAIN_SIZE = 25; // Relay risky: on my machine
													// it went up to size of 55
													// before reaching stack
													// overflow
	private static final String[] COMPLEXITY_STRINGS = { "No walls", "Easy", "Medium", "Hard", "Most complex" };

	private static final String[] ENTRANCE_STRINGS = { "Top left", "Top right", "Center", "Bottom left",
			"Bottom right" };

	// variables
	static private final int problemSize = 12;
	static private final String entranceLabel = "Top left";
	static private final String complexityLabel = "No walls";
	static private final int numberOfExits = 1;

	// graphical components
	private TextField numberOfExitsInput;
	private ScrollBar sc;
	private ScrollBar corridorWitdhInput;
	private ComboBox<String> entranceSelection;
	private ComboBox<String> complexitySelection;
	
	private GridPane layout;
	
	public FELabForm() {
		this.layout=new GridPane();
		this.getChildren().add(this.layout);
		initialise();
	}

	private void initialise() {
		//this.setPrefWidth(300);
		//this.setPadding(new Insets(8));

//		int numberOfColumns = 2;
//		for (int i = 0; i < numberOfColumns; i++) {
//			ColumnConstraints column = new ColumnConstraints();
//			column.setPercentWidth(100 / numberOfColumns);
//			this.layout.getColumnConstraints().add(column);
//		}
//
//		int numberOfRows = 6;
//		for (int i = 0; i < numberOfRows; i++) {
//			RowConstraints row = new RowConstraints();
//			row.setPercentHeight(100 / numberOfRows);
//			this.layout.getRowConstraints().add(row);
//		}

		this.layout.add(new Label("Labyrinth size: "), 0, 0);
		sc = new ScrollBar();
		sc.setMin(MIN_DOMAIN_SIZE);
		sc.setMax(MAX_DOMAIN_SIZE);
		sc.setValue(problemSize);
		sc.setUnitIncrement(1);
		sc.setBlockIncrement(1);
		this.layout.add(sc, 0, 1, 2, 1);

		this.layout.add(new Label("Entrance position: "), 0, 2);
		entranceSelection = new ComboBox<>();
		entranceSelection.getItems().addAll(ENTRANCE_STRINGS);
		entranceSelection.setValue(entranceLabel);
		this.layout.add(entranceSelection, 1, 2);

		this.layout.add(new Label("Labyrinth complexity: "), 0, 3);
		complexitySelection = new ComboBox<>();
		complexitySelection.getItems().addAll(COMPLEXITY_STRINGS);
		complexitySelection.setValue(complexityLabel);
		this.layout.add(complexitySelection, 1, 3);

		this.layout.add(new Label("Number of exits: "), 0, 4);
		numberOfExitsInput = new TextField();
		numberOfExitsInput.setText(String.valueOf(numberOfExits));
		this.layout.add(numberOfExitsInput, 1, 4);

		this.layout.add(new Label("Corridor width: "), 0, 5);
		corridorWitdhInput = new ScrollBar();
		corridorWitdhInput.setMin(0);
		corridorWitdhInput.setMax(1);
		corridorWitdhInput.setValue(0);
		corridorWitdhInput.setUnitIncrement(0.1);
		corridorWitdhInput.setBlockIncrement(0.1);
		this.layout.add(corridorWitdhInput, 0, 6, 2, 1);
	}

	public int getLabyrinthSizeInput() {
		return (int) sc.getValue();
	}

	public EEntranceLabel getEntranceLabelInput() {
		String inputLabel = entranceSelection.getValue();
		switch (inputLabel) {
		case "Top left":
			return EEntranceLabel.TOP_LEFT;
		case "Top right":
			return EEntranceLabel.TOP_RIGHT;
		case "Center":
			return EEntranceLabel.CENTER;
		case "Bottom left":
			return EEntranceLabel.BOTTOM_LEFT;
		case "Bottom right":
			return EEntranceLabel.BOTTOM_RIGHT;
		default:
			return EEntranceLabel.TOP_LEFT;
		}
	}

	public EComplexity getComplexityInput() {
		String inputLabel = complexitySelection.getValue();
		switch (inputLabel) {
		case "No walls":
			return EComplexity.NO_WALLS;
		case "Easy":
			return EComplexity.EASY;
		case "Medium":
			return EComplexity.MEDIUM;
		case "Hard":
			return EComplexity.HARD;
		case "Most complex":
			return EComplexity.MOST_COMPLEX;
		default:
			return EComplexity.MOST_COMPLEX;
		}
	}

	public int getNumberOfExitsInput() {
		//TODO check bad inputs
		return Integer.parseInt(numberOfExitsInput.getText());
	}

	@Override
	public IDomainParameters getAlgorithmParameters() {
		return new FELabParameters(this.getLabyrinthSizeInput(), this.getEntranceLabelInput(), this.getComplexityInput(), this.getNumberOfExitsInput());
	}

	public double getCorridorWidthInput() {
		return corridorWitdhInput.getValue();
	}

}
