package hr.fer.bachelorThesis.gui.labyrinth;

import java.util.HashMap;

import hr.fer.bachelorThesis.gui.IGUIController;
import hr.fer.bachelorThesis.labyrinth.ILabyrinth;
import hr.fer.bachelorThesis.labyrinth.util.DomainPaint;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.layout.Pane;
import javafx.scene.paint.Color;

public class LabyrinthView extends Pane {
	
	private int canvasHeight;
	private int canvasWidth;

	private Canvas labyrinthFloor;
	private HashMap<Long,Canvas> populationMemoryMap;
	private HashMap<Long,Canvas> agentsPositionsMap;
	private Canvas labyrinthWalls;
	private HashMap<Long,Canvas> agentMarkersMap;
	//private Canvas fog;
	private HashMap<Long,Canvas> agentVectorsMap;
	
	public LabyrinthView(IGUIController guiController,int canvasWidth,int canvasHeight) {
		//System.err.println("Constructing labyrinth view");
		this.canvasHeight=canvasHeight;
		this.canvasWidth=canvasWidth;
		this.populationMemoryMap=new HashMap<>();
		this.agentsPositionsMap=new HashMap<>();
		this.agentMarkersMap=new HashMap<>();
		this.agentVectorsMap=new HashMap<>();
		generateLayers();
	}
	
	private void generateLayers() {

		labyrinthFloor= new Canvas(canvasWidth, canvasHeight);
//		labyrinthFloor= new Canvas();
		
		Canvas populationMemory0= new Canvas(canvasWidth, canvasHeight);
		Canvas populationMemory1= new Canvas(canvasWidth, canvasHeight);
		this.populationMemoryMap.put(0L, populationMemory0);
		this.populationMemoryMap.put(1L, populationMemory1);
		
		Canvas agentsPositions0= new Canvas(canvasWidth, canvasHeight);
		Canvas agentsPositions1= new Canvas(canvasWidth, canvasHeight);
		this.agentsPositionsMap.put(0L, agentsPositions0);
		this.agentsPositionsMap.put(1L, agentsPositions1);
		
		labyrinthWalls= new Canvas(canvasWidth, canvasHeight);
		
		Canvas agentMarkers0= new Canvas(canvasWidth, canvasHeight);
		Canvas agentMarkers1= new Canvas(canvasWidth, canvasHeight);
		this.agentMarkersMap.put(0L, agentMarkers0);
		this.agentMarkersMap.put(1L, agentMarkers1);

		//fog=new Canvas(canvasWidth, canvasHeight);
		
		Canvas agentVectors0= new Canvas(canvasWidth, canvasHeight);
		Canvas agentVectors1= new Canvas(canvasWidth, canvasHeight);
		this.agentVectorsMap.put(0L, agentVectors0);
		this.agentVectorsMap.put(1L, agentVectors1);
		

		
		this.getChildren().removeAll();
		this.getChildren().add(labyrinthFloor);
		this.getChildren().add(populationMemory0);
		this.getChildren().add(populationMemory1);
		this.getChildren().add(agentsPositions0);
		this.getChildren().add(agentsPositions1);
		this.getChildren().add(labyrinthWalls);
		this.getChildren().add(agentMarkers0);
		this.getChildren().add(agentMarkers1);
		//this.getChildren().add(fog);
		this.getChildren().add(agentVectors0);
		this.getChildren().add(agentVectors1);
		//this.setStyle("-fx-background-color: #A9A9A9 ");
		this.setStyle("-fx-background-color: #000000 ");
	}


	public void setLabyrinth(ILabyrinth chosenLabyrinth) {
		//clear all layers
		this.clearAllAlgorithmLayers();
		this.clearLabyinrthLayers();
		//redraw domain
		
		chosenLabyrinth.drawDomain(new DomainPaint(Color.BLACK, Color.DARKGREY, Color.RED, Color.GREENYELLOW), labyrinthFloor.getGraphicsContext2D(), labyrinthWalls.getGraphicsContext2D());

	}
	
	private void clearLabyinrthLayers() {
		clearLayer(this.labyrinthFloor);
		clearLayer(this.labyrinthWalls);
	}

	public void clearAllAlgorithmLayers(){
		//clearLayer(this.labyrinthFloor);
		for(Canvas c:this.populationMemoryMap.values())
			clearLayer(c);
		for(Canvas c:this.agentsPositionsMap.values())
			clearLayer(c);
		//clearLayer(this.labyrinthWalls);
		for(Canvas c:this.agentMarkersMap.values())
			clearLayer(c);
		//clearLayer(this.fog);
		for(Canvas c:this.agentVectorsMap.values())
			clearLayer(c);
		
		
		
	}
	
	/**
	 * Method removes all graphics from the given canvas
	 * 
	 * @param layer
	 *            canvas to be cleared
	 */
	private void clearLayer(Canvas layer) {
		GraphicsContext layer_g = layer.getGraphicsContext2D();
		layer_g.clearRect(0, 0, layer.getWidth(), layer.getHeight());
	}
	
	public GraphicsContext genericFetcher(long id,HashMap<Long,Canvas> map, int insertionIndex){
		if(map.containsKey(id)){
			return map.get(id).getGraphicsContext2D();
		}else{
			//Canvas canvas=new Canvas(canvasWidth, canvasHeight);
			Canvas canvas=new Canvas( );
			if(insertionIndex>this.getChildren().size()){
				this.getChildren().add(canvas);
				
			}else{
				this.getChildren().add(insertionIndex, canvas);
				
			}
			map.put(id, canvas);
			return canvas.getGraphicsContext2D();
		}
	}
	
	public GraphicsContext getPopulationMemoryLayer(long id){
		return genericFetcher(id, this.populationMemoryMap, 1+this.populationMemoryMap.size());
	}
	
	public GraphicsContext getAgentsPotsitionsLayer(long id){
		return genericFetcher(id, this.agentsPositionsMap, 1+this.populationMemoryMap.size()+this.agentsPositionsMap.size());
	}
	
	public GraphicsContext getAgentsMarkersLayer(long id){
		return genericFetcher(id, this.agentMarkersMap, 1+this.populationMemoryMap.size()+this.agentsPositionsMap.size()+this.agentMarkersMap.size());
	}
	
	public GraphicsContext getagentsVectorsLayer(long id){
		return genericFetcher(id, this.agentVectorsMap, 1+this.populationMemoryMap.size()+this.agentsPositionsMap.size()+this.agentMarkersMap.size()+this.agentVectorsMap.size());
	}

	public void cleanAgentCanvas(long id) {
		this.clearIterationLayers(id);
	}

	private void clearIterationLayers(long id) {
		//TODO, skip memory to keep all movement on one canvas:
		//clearLayer(this.populationMemoryMap.get(id));
		clearLayer(this.agentsPositionsMap.get(id));
		clearLayer(this.agentMarkersMap.get(id));
		clearLayer(this.agentVectorsMap.get(id));
	}

//	private void clearIterationLayers() {
//		//clearLayer(this.labyrinthFloor);
//		for(Canvas c:this.populationMemoryMap.values())
//			clearLayer(c);
//		for(Canvas c:this.agentsPositionsMap.values())
//			clearLayer(c);
//		//clearLayer(this.labyrinthWalls);
//		for(Canvas c:this.agentMarkersMap.values())
//			clearLayer(c);
//		//clearLayer(this.fog);
//		for(Canvas c:this.agentVectorsMap.values())
//			clearLayer(c);
//	}
	
	public int getCanvasWidth(){
		return this.canvasWidth;
	}
	
	public int getCanvasHeight(){
		return this.canvasHeight;
	}
	
	
}
