package hr.fer.bachelorThesis.labyrinth;

import java.util.List;

import hr.fer.bachelorThesis.labyrinth.util.DomainPaint;
import hr.fer.bachelorThesis.util.linearAlgebra.IVector;
import javafx.scene.canvas.GraphicsContext;

/**
 * Interface for a object that is traversable as a labyrinth.
 * @author Viran
 *
 */
public interface ILabyrinth {


	/**
	 * Method is used to draw the visual representation of the domain
	 * @param paintElements Preferred colours for the domain elements.
	 * @param background_g GraphicsContext used to draw the background of domain
	 * @param front_g GraphicsContext used to draw the walls, start point and exit points
	 */
	void drawDomain(DomainPaint paintElements,GraphicsContext background_g, GraphicsContext front_g);

	/**
	 * Method returns the value of the domain at the given point T
	 * @param point where the domain value is being calculated
	 * @return value of the domain at the given point
	 */
	public ELabyrinthElement valueAt(IVector point);
	
	/**
	 * Returns number of rows in domain
	 */
	public int getRowCount();

	/**
	 * Returns number of columns in domain
	 */
	public int getColumnCount();

	/**
	 * Returns block height
	 */
	public double getBlockHeight();
	
	/**
	 * Returns block width
	 */
	public double getBlockWidth();
	
	/**
	 * Method returns the starting position in the given domain
	 * @return starting position
	 */
	public IVector getStartingPosition();
	
	/**
	 * For the current position and the direction vector, return a list of new position the agent will pass while moving.
	 * This method rescales the direction vector if such a change is needed.
	 * @param curPos Current position.
	 * @param directionVector Direction vector from position.
	 * @return List of positions the agent must step on-
	 */
	public List<IVector> generatePosition(IVector curPos, IVector directionVector);

	/**
	 * Tell if the current position is the agent is done with its search one found the given position.
	 * @param curPos Position he checks.
	 * @return True if this is the agent can stop, false otherwise.
	 */
	boolean isFinished(IVector curPos);

	boolean inDomain(IVector position);
}
