package hr.fer.bachelorThesis.labyrinth;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import hr.fer.bachelorThesis.labyrinth.generator.IComplexityModifier;
import hr.fer.bachelorThesis.labyrinth.generator.IExitAttractionStrategy;
import hr.fer.bachelorThesis.labyrinth.generator.ILabyrinthGeneratorStrategy;
import hr.fer.bachelorThesis.labyrinth.util.DomainPaint;
import hr.fer.bachelorThesis.util.linearAlgebra.IVector;
import hr.fer.bachelorThesis.util.linearAlgebra.IncompatibleOperandException;
import hr.fer.bachelorThesis.util.linearAlgebra.Vector;
import javafx.scene.canvas.GraphicsContext;


public class Labyrinth implements ILabyrinth{
	
	private double blockSize;
	
	//useful elements

	private IVector startingPosition;
	private IVector[] endingPosition;
	
	//matrix elements
	private int rows;
	private int cols;
	private LabyrinthElement[][] labyrinthMatrix;
	private IExitAttractionStrategy exitAttStrategy;
	
	public Labyrinth(int labyrinthSize, int canvasDimension, ILabyrinthGeneratorStrategy genStrategy, IComplexityModifier complexytiModifier, IExitAttractionStrategy exitAttStrategy) {
		int labyrinthDimension=convertToDim(labyrinthSize);
		
		this.rows=labyrinthDimension;
		this.cols=labyrinthDimension;
		this.exitAttStrategy=exitAttStrategy;
		this.blockSize=((double)canvasDimension)/labyrinthDimension;
		
		this.labyrinthMatrix=new LabyrinthElement[rows][cols];
		for(int i=0;i<rows;i++)
			for(int j=0;j<cols;j++)
				this.labyrinthMatrix[i][j]=new LabyrinthElement(ELabyrinthElement.PASSABLE);
		List<IVector> markedPositions=genStrategy.generateLabyrinth(this.labyrinthMatrix,this.rows,this.cols);
		this.startingPosition=markedPositions.get(0);
		for(int i=1;i<markedPositions.size();i++)
			endingPosition[i-1]=markedPositions.get(i);
		
		complexytiModifier.modifyLabyrinth(labyrinthMatrix,rows,cols);
		
	}
	

	@SuppressWarnings("unused")
	private static void printGrid(LabyrinthElement[][] labyrinthMatrix, int rows, int cols) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < rows; i++) {
			for (int j = 0; j < cols; j++) {
				switch (labyrinthMatrix[i][j].get()) {
				case EXIT:
					sb.append("E ");
					break;
				case PASSABLE:
					sb.append("_ ");
					break;
				case START:
					sb.append("S ");
					break;
				case WALL:
					sb.append("# ");
					break;
				default:
					sb.append("? ");
				
				}
			}
			sb.append("\n");
		}
		//System.out.println(sb.toString());
	}
	
	private int convertToDim(int labyrinthSize) {
		int dim=1;
		while(true){
			dim+=2;
			if(isPrime(dim)) continue;
			labyrinthSize--;
			if(labyrinthSize==0)
				break;
		}
		return dim;
	}
	
	boolean isPrime(int n) {
	    if(n < 2) return false;
	    if(n == 2 || n == 3) return true;
	    if(n%2 == 0 || n%3 == 0) return false;
	    long sqrtN = (long)Math.sqrt(n)+1;
	    for(long i = 6L; i <= sqrtN; i += 6) {
	        if(n%(i-1) == 0 || n%(i+1) == 0) return false;
	    }
	    return true;
	}

	@Override
	public void drawDomain(DomainPaint paintElements,GraphicsContext background_g, GraphicsContext front_g) {

		for (int i = 0; i < rows; ++i) {
			for (int j = 0; j < cols; ++j) {
				
				background_g.setFill(paintElements.getFloorColor());
				background_g.fillRect(j * blockSize, i * blockSize, blockSize, blockSize);

				if (labyrinthMatrix[i][j].equals(ELabyrinthElement.WALL)) {

					front_g.setFill(paintElements.getWallColor());
					front_g.fillRect(j * blockSize, i * blockSize, blockSize, blockSize);
					
				} else if (labyrinthMatrix[i][j].equals(ELabyrinthElement.START)) {

					front_g.setFill(paintElements.getStartColor());
					front_g.fillOval(j * blockSize, i * blockSize, blockSize, blockSize);
					
					background_g.setGlobalAlpha(0.5);
					background_g.fillOval(j * blockSize, i * blockSize, blockSize*1.5, blockSize*1.5);
					background_g.setGlobalAlpha(1);
					
				} else if (labyrinthMatrix[i][j].equals(ELabyrinthElement.EXIT)){

					front_g.setFill(paintElements.getExitColor());
					front_g.fillOval(j * blockSize, i * blockSize, blockSize, blockSize);
					
					background_g.setGlobalAlpha(0.5);
					background_g.fillOval(j * blockSize, i * blockSize, blockSize*1.5, blockSize*1.5);
					background_g.setGlobalAlpha(1);					
					
				}
			}
		}
		
	}
	
//	@Override
//	public void drawDomain(DomainPaint paintElements,GraphicsContext background_g, GraphicsContext front_g) {
//		for (int i = 0; i < rows; ++i) {
//			for (int j = 0; j < cols; ++j) {
//				
//				background_g.setFill(paintElements.getFloorColor());
//				background_g.fillRect(i * blockSize, j * blockSize, blockSize, blockSize);
//
//				if (labyrinthMatrix[i][j].equals(ELabyrinthElement.WALL)) {
//
//					front_g.setFill(paintElements.getWallColor());
//					front_g.fillRect(i * blockSize, j * blockSize, blockSize, blockSize);
//					
//				} else if (labyrinthMatrix[i][j].equals(ELabyrinthElement.START)) {
//
//					front_g.setFill(paintElements.getStartColor());
//					front_g.fillOval(i * blockSize, j * blockSize, blockSize, blockSize);
//					
//					background_g.setGlobalAlpha(0.5);
//					background_g.fillOval(i * blockSize, j * blockSize, blockSize*1.5, blockSize*1.5);
//					background_g.setGlobalAlpha(1);
//					
//				} else if (labyrinthMatrix[i][j].equals(ELabyrinthElement.EXIT)){
//
//					front_g.setFill(paintElements.getExitColor());
//					front_g.fillOval(i * blockSize, j * blockSize, blockSize, blockSize);
//					
//					background_g.setGlobalAlpha(0.5);
//					background_g.fillOval(i * blockSize, j * blockSize, blockSize*1.5, blockSize*1.5);
//					background_g.setGlobalAlpha(1);					
//					
//				}
//			}
//		}
//		
//	}
	
	@Override
	public ELabyrinthElement valueAt(IVector point) {
		return this.labyrinthMatrix[(int)point.get(0)][(int)point.get(1)].get();
	}
	
	@Override
	public int getRowCount() {
		return this.rows;
	}
	
	@Override
	public int getColumnCount() {
		return this.cols;
	}
	
	@Override
	public double getBlockHeight() {
		return this.blockSize; //TODO forced cast to integer blockSize
	}
	
	@Override
	public double getBlockWidth() {
		return this.blockSize; //TODO
	}
	
	@Override
	public IVector getStartingPosition() {
		return this.startingPosition;
	}

	@Override
	public boolean isFinished(IVector curPos) {
		return this.valueAt(curPos)==ELabyrinthElement.EXIT;
	}

	
	@Override
	public List<IVector> generatePosition(IVector curPos, IVector directionVector) {
		IVector endPoint=null;
		
		try {
			endPoint=curPos.nAdd(directionVector);
		} catch (IncompatibleOperandException ignorable) {
			// TODO Auto-generated catch block
		}
		
		int xs=(int)Math.ceil(curPos.get(0));
		int ys=(int)Math.ceil(curPos.get(1));
		int xe=(int)Math.ceil(endPoint.get(0));
		int ye=(int)Math.ceil(endPoint.get(1));
		xs=xs<0?0:xs;
		ys=ys<0?0:ys;
		xe=xe<0?0:xe;
		ye=ye<0?0:ye;
		
		//System.out.print(xs+","+ys+"->"+xe+","+ye+"  ");
		
		List<IVector> list=new LinkedList<>();
		if(xs<=xe){
			if(ys<=ye){
				getPos90(list,xs,ys,xe,ye);
			}else{
				getNeg90(list,xs,ys,xe,ye);
			}
		}else{
			if(ys>=ye){
				getPos90(list,xe,ye,xs,ys);
			}else{
				getNeg90(list,xe,ye,xs,ys);
			}
		}
		if(!list.get(0).equals(curPos)){
			Collections.reverse(list);
		}
		//System.out.println(" "+list);
		double steps=list.size();
		for(int i=0;i<list.size();i++){
			IVector vec=list.get(i);
			int x=(int)vec.get(0);
			int y=(int)vec.get(1);
			if (isWall(x,y)){
				list=list.subList(0, i);
				//directionVector.normalise();
				break;			
			}
			if(isExit(x, y) ){
				list=list.subList(0, i+1);
				break;
			}
		}
		
		double coverdTotalRatio=((double)list.size())/steps;
		
				
		//shorten the distance vector if it's full length hasn't been utilised
		directionVector.scalarMultiply(coverdTotalRatio);
		return list;
	}
	

	private boolean isWall(int x, int y) {
		return labyrinthMatrix[x][y].equals(ELabyrinthElement.WALL);
	}


	private void  getNeg90(List<IVector> list,int xs, int ys, int xe, int ye) {
		int x,yc,correction;
		int a,yf;

		
		if(-(ye-ys)<= (xe-xs) ){
			a=(ye-ys)*2;
			yc=ys; yf=xe-xs; correction= (xe-xs)*2;
			for(x=xs;x<=xe;x++){
				
				//CORE PART: if at any moment wall is detected, stop adding the points to list
				//but continue to count all steps you could possibly take
				
				if(!outOfBorder(x,yc)){
					list.add(new Vector(x,yc));
				}
				//CORE PART DONE
				yf+=a;
				if(yf<=0){
					yf+=correction;
					yc-=1;
				}
			}
		}else{
			x=xe; xe=ys; ys=x;
			x=xs; xs=ye; ye=x; 
			
			a=(ye-ys)*2;
			yc=ys; yf=xe-xs; correction= (xe-xs)*2;
			for(x=xs;x<=xe;x++){

				//CORE PART: if at any moment wall is detected, stop adding the points to list
				//but continue to count all steps you could possibly take
				
				if(!outOfBorder(yc,x)){
					list.add(new Vector(yc,x));
				}
				//CORE PART DONE

				
				yf+=a;
				if(yf<=0){
					yf+=correction;
					yc-=1;
				}
			}
		}
	}


	private void  getPos90(List<IVector> list,int xs, int ys, int xe, int ye) {
		int x,yc,correction;
		int a,yf;
		
		
		
		if(ye-ys <= xe-xs){
			a=(ye-ys)*2;
			yc=ys; yf=-(xe-xs); correction= (xe-xs)*-2;
			for(x=xs;x<=xe;x++){
				
				//CORE PART: if at any moment wall is detected, stop adding the points to list
				//but continue to count all steps you could possibly take
				
				if(!outOfBorder(x,yc)){
					list.add(new Vector(x,yc));
				}
				//CORE PART DONE
				yf+=a;
				if(yf>=0){
					yf+=correction;
					yc+=1;
				}
			}
		}else{
			x=xe; xe=ye; ye=x;
			x=xs; xs=ys; ys=x;
			
			a=(ye-ys)*2;
			yc=ys; yf=-(xe-xs); correction= (xe-xs)*-2;
			for(x=xs;x<=xe;x++){
				
				//CORE PART: if at any moment wall is detected, stop adding the points to list
				//but continue to count all steps you could possibly take
				
				
				if(!outOfBorder(yc,x)){
					list.add(new Vector(yc,x));
				}
				//CORE PART DONE
				
				yf+=a;
				if(yf>=0){
					yf+=correction;
					yc+=1;
				}
			}
		}
	}

	private boolean outOfBorder(int x, int y) {
		return (x>=this.rows || x<0 || y>=this.cols || y<0);
	}


	private boolean isExit(int x, int y) {
		//if(x>=this.rows || x<0 || y>=this.cols || y<0) return false;
		
		return labyrinthMatrix[x][y].equals(ELabyrinthElement.EXIT);
	}
	
	
	@Override
	public String toString() {
		StringBuffer sb= new StringBuffer();
		
		for(int i=0;i<rows;i++){
			for(int j=0;j<cols;j++){
				sb.append(this.labyrinthMatrix[i][j].toString().charAt(0)+" ");
			}
			sb.append("\n");
		}
		
		return sb.toString();
	}


	@Override
	public boolean inDomain(IVector position) {
		return (position.get(0)>=0 && position.get(0)<rows && position.get(1)>=0 && position.get(1)<cols);
	}

}
