package hr.fer.bachelorThesis.labyrinth.generator;

import java.util.LinkedList;
import java.util.Random;

import hr.fer.bachelorThesis.labyrinth.ELabyrinthElement;
import hr.fer.bachelorThesis.labyrinth.LabyrinthElement;
import hr.fer.bachelorThesis.util.linearAlgebra.IVector;
import hr.fer.bachelorThesis.util.linearAlgebra.Vector;

public class WallDeletingComplexityModifier implements IComplexityModifier {

	private EComplexity complexity;
	private int exitRadius;

	public WallDeletingComplexityModifier(EComplexity complexity, int exitProtectionRadius) {
		this.complexity = complexity;
		this.exitRadius = exitProtectionRadius;
	}

	@Override
	public void modifyLabyrinth(LabyrinthElement[][] labyrinthMatrix, int rows, int cols) {
		LinkedList<IVector> exitPositions = new LinkedList<>();
		LinkedList<IVector> wallPositions = getWallPositionList(labyrinthMatrix, rows, cols, exitPositions);

		if (this.complexity != EComplexity.NO_WALLS)
			removeExitSurroundingWalls(wallPositions, exitPositions);

		int wallsToRemove = calcNumOfRemovedWalls(wallPositions.size());
		Random rand = new Random();
		int x, y;
		while (wallsToRemove > 0 && !wallPositions.isEmpty()) {
			IVector wallPos = wallPositions.remove(rand.nextInt(wallPositions.size()));

			x = (int) wallPos.get(0);
			y = (int) wallPos.get(1);

			if(this.complexity == EComplexity.NO_WALLS){
				labyrinthMatrix[x][y].set(ELabyrinthElement.PASSABLE);
				wallsToRemove--;
				
			}else if (allowedToRemove(labyrinthMatrix, rows, cols, x, y) ) {
				labyrinthMatrix[x][y].set(ELabyrinthElement.PASSABLE);
				wallsToRemove--;
			}
			
		}
	}

	private boolean allowedToRemove(LabyrinthElement[][] labyrinthMatrix, int rows, int cols, int x, int y) {
		if (x - 1 >= 0 && x + 1 < rows && y - 1 >= 0 && y + 1 < cols) {
			if (labyrinthMatrix[x - 1][y].get() == ELabyrinthElement.WALL
					&& labyrinthMatrix[x + 1][y].get() == ELabyrinthElement.WALL
					&& labyrinthMatrix[x][y - 1].get() != ELabyrinthElement.WALL
					&& labyrinthMatrix[x][y + 1].get() != ELabyrinthElement.WALL) {
				return true;
			}
			if (labyrinthMatrix[x - 1][y].get() != ELabyrinthElement.WALL
					&& labyrinthMatrix[x + 1][y].get() != ELabyrinthElement.WALL
					&& labyrinthMatrix[x][y - 1].get() == ELabyrinthElement.WALL
					&& labyrinthMatrix[x][y + 1].get() == ELabyrinthElement.WALL) {
				return true;
			}
		}
		// wall is on the border
		return false;
	}

	private void removeExitSurroundingWalls(LinkedList<IVector> wallPositions, LinkedList<IVector> exitPositions) {
		int eX, eY;
		for (IVector exit : exitPositions) {
			eX = (int) exit.get(0);
			eY = (int) exit.get(1);

			for (int i = -exitRadius; i <= exitRadius; i++) {
				for (int j = -exitRadius; j <= exitRadius; j++) {

					for (int wIndex = 0; wIndex < wallPositions.size(); wIndex++) {
						IVector wall = wallPositions.get(wIndex);
						if (wall.get(0) == (eX + i) && wall.get(1) == (eY + j))
							wallPositions.remove(wIndex);
					}

				}
			}

		}
	}

	private int calcNumOfRemovedWalls(int totalWalls) {
		switch (complexity) {
		case NO_WALLS:
			return (int) (totalWalls * WDCModifierConstants.NO_WALLS_PERCENT);
		case EASY:
			return (int) (totalWalls * WDCModifierConstants.EASY_PERCENT);
		case MEDIUM:
			return (int) (totalWalls * WDCModifierConstants.MEDIUM_PERCENT);
		case HARD:
			return (int) (totalWalls * WDCModifierConstants.HARD_PERCENT);
		case MOST_COMPLEX:
			return (int) (totalWalls * WDCModifierConstants.MOST_COMPLEX_PERCENT);
		default:
			return 0;
		}
	}

	private LinkedList<IVector> getWallPositionList(LabyrinthElement[][] labyrinthMatrix, int rows, int cols,
			LinkedList<IVector> exitPositions) {
		LinkedList<IVector> wallPositions = new LinkedList<>();

		// not counting boarder walls
		for (int i = 1; i < rows - 1; i++) {
			for (int j = 1; j < cols - 1; j++) {
				if (labyrinthMatrix[i][j].equals(ELabyrinthElement.WALL))
					wallPositions.add(new Vector(i, j));
				if (labyrinthMatrix[i][j].equals(ELabyrinthElement.EXIT))
					exitPositions.add(new Vector(i, j));

			}
		}

		return wallPositions;
	}

}
