package hr.fer.bachelorThesis.algorithm.strategy;

import hr.fer.bachelorThesis.algorithm.pso.IAgent;
import hr.fer.bachelorThesis.util.linearAlgebra.IVector;
import hr.fer.bachelorThesis.util.linearAlgebra.IncompatibleOperandException;

public class TargetDirectionStrategy implements IAgentStrategy {

	private IAgentStrategy strategy;
	private double maxVectorSize;

	public TargetDirectionStrategy(IAgentStrategy strategy,double maxVectorSize) {
		this.strategy=strategy;
		this.maxVectorSize=maxVectorSize;
	}
	
	@Override
	public IVector nextDirectionVector(IAgent agent) throws IncompatibleOperandException {
		if(agent.getPersonalTargetPosition()!=null){

			// get the direction vector by subtracting the best position with the
			// current position
			IVector thisStrategyVector = agent.getPersonalTargetPosition().nSub(agent.getCurrentPosition());
			// TODO make the max size to this position dynamic-a bit more complex
			// exploiting the information
			thisStrategyVector.scalarMultiply(this.maxVectorSize);

			//System.out.println("Target st. dir vector: "+thisStrategyVector);
			return strategy.nextDirectionVector(agent).add(thisStrategyVector);	
		}else{
			return strategy.nextDirectionVector(agent);		
		}
		
	}

}
